# Author::    Alessio Caiazza  (mailto:nolith@abisso.org)
# Copyright:: Copyright (c) 2010 Alessio Caiazza
# License::   New BSD License - http://www.opensource.org/licenses/bsd-license.php
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following
#   disclaimer in the documentation and/or other materials provided
#   with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

require 'securerandom'
require 'openssl'
require 'bit-struct'

# 1-complement is uses to compute crc field
#
#
class Numeric
    def ones_complement(bits)
        self ^ ((1 << bits) - 1)
    end
end

module YubiRuby
  #
  # This class simulate a yubikey allowing you to make some tests
  #
  class FakeYubikey
    # Maximum allowed length for a modhex encoded public id
    PUBLIC_ID_MAX_LEN = 32

    #
    # Yubikey internal data
    #
    class Data < BitStruct
      YUBIKEY_BLOCK_SIZE = 16
      YUBIKEY_KEY_SIZE = 16
      YUBIKEY_UID_SIZE = 6
      SESSION_COUNTER_OVERFLOW = 0x7FFF

      hex_octets  :uid,     YUBIKEY_UID_SIZE*8,     "User ID"
      unsigned    :session_counter,    16, {:endian=>:little, :display_name=>"Session Counter"}
      unsigned    :timestamp_low,   16, {:endian=>:little, :display_name=>"Timestamp low"}
      unsigned    :timestamp_high,  8, "Timestamp high"
      unsigned    :session_use,  8, "Session Use"
      unsigned    :random,   16,  {:endian=>:little, :display_name=>"random"}
      unsigned    :crc,  16, {:endian=>:little, :display_name=>"crc"}

    end

    attr_accessor :key
	attr_reader :public_id, :data

    def initialize
      @public_id = ""
      @data = YubiRuby::FakeYubikey::Data.new
      @data.session_counter = 0
      init
    end

    #
    # Sets the public id of the key.
    #
    # public_id must be a modhex valid string no longer than
    # 32 char, if longer it will be truncated.
    #
    # @param [String, #read] value the new public_id
    # @return [String] the new public_id
    def public_id=(value)
      raise NoModHexEncodedError unless value.modhex?

      value = value[0, PUBLIC_ID_MAX_LEN] if value.length > PUBLIC_ID_MAX_LEN
      raise NoModHexEncodedError unless value.modhex?

      @public_id = value
    end

    # Simaluates the power on sequence of a YubuKey.
    #
    # This will increase session_counter
    def init
      @data.session_counter += 1
      if (@data.session_counter == Data::SESSION_COUNTER_OVERFLOW+1)
        raise YubikeySessionCounterOverflow.new
      end
      @data.session_use = 0
      @data.timestamp_low = rand(2**16)
      @data.timestamp_high = rand(2**8)
    end

    %w{session_use timestamp_low timestamp_high session_counter random crc}.each do |name|
      define_method name do
        @data.send name
      end
    end

    # Generates a random key attribute
    def set_random_key
      @key = produce_random_bytes_in_hex Data::YUBIKEY_KEY_SIZE
    end

    # Generates a random uid attribute
    def set_random_uid
      self.uid= produce_random_bytes_in_hex(Data::YUBIKEY_UID_SIZE)
    end

    # Sets the attribute uid
    def uid=(value)
      if YubiRuby::HEX.decode(value).size == Data::YUBIKEY_UID_SIZE
        @data.uid = "#{value[0,2]}:#{value[2,2]}:#{value[4,2]}:#{value[6,2]}:#{value[8,2]}:#{value[10,2]}"
      else
        raise NoHexEncodedError.new
      end
    end

    # Returns the value of attribute uid.
    def uid
      @data.uid.gsub(':','')
    end

    # Simulates the pressing of the yubikey's button
    #
    # session_use will increase and if it overflows also
    # session_counter will be increased
    def otp
      @data.random = rand(2**16)
      @data.session_use += 1
      @data.session_counter +=1 if @data.session_use == 0
      generate_otp
    end

    def to_s
      @data.to_s
    end

    def self.generate_specific_otp_and_fake_yubikey(key, uid, session_counter,
        timestamp_low, timestamp_high, session_use, random)
      y_key = FakeYubikey.new
      y_key.key = key
      y_key.uid = uid
      y_key.data.session_use = session_use
      y_key.data.timestamp_low = timestamp_low
      y_key.data.timestamp_high = timestamp_high
      y_key.data.session_counter = session_counter
      y_key.data.random = random
      [y_key.generate_otp, y_key]
    end

    # Generates an OTP without altering any internal data.
    def generate_otp
      @cipher = OpenSSL::Cipher.new( "AES-128-ECB" ) unless @cipher
      @cipher.encrypt
      @cipher.key = YubiRuby::HEX.decode( @key )

      @data.crc = YubiRuby::CRC16.calculate(@data.to_s[0...-2]).ones_complement 16
      data = @cipher.update( @data.to_s ).modhex_encode

      "#{@public_id}#{data}"

	ensure
		@cipher.reset
    end

    #serialization support
    def marshal_dump
        {'key' => key, 'uid' => uid, 'session_counter' => @data.session_counter,
          'public_id' => public_id }
    end

    #serialization support
    def marshal_load(data)
      @data = YubiRuby::FakeYubikey::Data.new
      self.key = data['key']
      self.uid = data['uid']
      self.data.session_counter = data['session_counter']
      self.public_id = data['public_id']
      init
    end


    private
    def produce_random_bytes_in_hex(size)
      YubiRuby::HEX.encode( SecureRandom.random_bytes(size) )
    end
  end
end
