# Author::    Alessio Caiazza  (mailto:nolith@abisso.org)
# Copyright:: Copyright (c) 2010 Alessio Caiazza
# License::   New BSD License - http://www.opensource.org/licenses/bsd-license.php
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following
#   disclaimer in the documentation and/or other materials provided
#   with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


module YubiRuby
  # Hex string encoder/decoder
  #
  # You may consider extending String with YubiRuby::HEX
  #   class String
  #     include YubiRuby::HEX
  #   end
  #
  #   "foo bar".hex_encode        #=> "666f6f20626172"
  #   "666f6f20626172".hex_decode #=> "foo bar"
  module HEX

    # call-seq:
    #   YubiRuby::HEX.encode("string") -> "hex string"
    #
    # Encodes <tt>obj.to_str</tt> into an <tt>hex string</tt>.
    #
    def self.encode( obj )
	  return obj.unpack( 'H*' ).first
    end

    # call-seq:
    #   hex_encode -> "hex string"
    #
    # Invokes YubiRuby::HEX.encode on +self+.
    def hex_encode
      HEX.encode(self)
    end

    # call-seq:
    #   YubiRuby::HEX.decode("hex string") -> "string"
    #
    # Decodes <tt>obj.to_str</tt> into a <tt>string</tt>.
    #
    def self.decode( obj )
      raise NoHexEncodedError unless self.hex?( obj )
      return obj.to_s.scan(/[[:xdigit:]]{2}/).map{|h| h.hex.chr }.join
    end

    # call-seq:
    #   hex_decode -> "string"
    #
    # Invokes YubiRuby::HEX.decode on +self+.
    def hex_decode
      HEX.decode(self)
    end

    # call-seq:
    #   YubiRuby::HEX.hex?("hex string") -> true or false
    #
    # Tests if <tt>obj.to_str</tt> is a valid a <tt>hex string</tt>.
    #
    # An <tt>hex string</tt> length must be pair.
    def self.hex?( obj )
      s = obj.to_str
      return false unless (s.length % 2 == 0)
      return (s.upcase =~ /[^A-F0-9]/).nil?
    end

    # call-seq:
    #   hex? -> true or false
    #
    # Invokes YubiRuby::HEX.hex? on +self+.
    def hex?
      HEX.hex?(self)
    end
  end
end
